@@ -2,12 +2,12 @@
 #
 # Picard, the next-generation MusicBrainz tagger
 #
-# Copyright (C) 2014-2015, 2018-2019 Laurent Monin
+# Copyright (C) 2014-2015, 2018-2021 Laurent Monin
 # Copyright (C) 2015 Rahul Raturi
 # Copyright (C) 2016 Ville Skyttä
 # Copyright (C) 2016 Wieland Hoffmann
 # Copyright (C) 2017 Sambhav Kothari
-# Copyright (C) 2019-2020 Philipp Wolfer
+# Copyright (C) 2019-2021 Philipp Wolfer
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -29,10 +29,8 @@ from collections import (
     namedtuple,
 )
 
-from picard import (
-    config,
-    log,
-)
+from picard import log
+from picard.config import get_config
 from picard.coverart.providers.caa import CoverArtProviderCaa
 from picard.coverart.providers.caa_release_group import (
     CoverArtProviderCaaReleaseGroup,
@@ -54,8 +52,10 @@ _cover_art_providers = ExtensionPoint(label='cover_art_providers')
 def register_cover_art_provider(provider):
     _cover_art_providers.register(provider.__module__, provider)
     if hasattr(provider, 'OPTIONS') and provider.OPTIONS:
-        provider.OPTIONS.NAME = provider.name.lower().replace(' ', '_')
-        provider.OPTIONS.TITLE = provider.title
+        if not hasattr(provider.OPTIONS, 'NAME'):
+            provider.OPTIONS.NAME = provider.name.lower().replace(' ', '_')
+        if not hasattr(provider.OPTIONS, 'TITLE'):
+            provider.OPTIONS.TITLE = provider.title
         register_options_page(provider.OPTIONS)
 
 
@@ -68,6 +68,7 @@ POrderTuple = namedtuple('OrderTuple', 'name position enabled')
 def cover_art_providers():
     def from_ca_providers_option():
         """Iterate through ca_providers option and yield name, position and enabled"""
+        config = get_config()
         for pos, (name, enabled) in enumerate(config.setting['ca_providers']):
             yield POrderTuple(name=name, position=pos, enabled=enabled)
 
@@ -84,7 +85,7 @@ def cover_art_providers():
         checked = 'x' if order[p.name].enabled else ' '
         return "%s [%s]" % (p.name, checked)
 
-    log.debug("CA Providers order: %s", ' > '.join([label(p) for p in ordered_providers]))
+    log.debug("CA Providers order: %s", ' > '.join(label(p) for p in ordered_providers))
 
     for p in ordered_providers:
         yield ProviderTuple(name=p.name, title=p.title, enabled=order[p.name].enabled, cls=p)
