@@ -1,7 +1,14 @@
 # -*- coding: utf-8 -*-
 #
 # Picard, the next-generation MusicBrainz tagger
-# Copyright (C) 2012 Michael Wiencek
+#
+# Copyright (C) 2012-2013 Michael Wiencek
+# Copyright (C) 2013, 2018-2020 Laurent Monin
+# Copyright (C) 2014, 2017 Sophist-UK
+# Copyright (C) 2017 Wieland Hoffmann
+# Copyright (C) 2017-2018 Sambhav Kothari
+# Copyright (C) 2018 Vishal Choudhary
+# Copyright (C) 2019, 2021 Philipp Wolfer
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,21 +24,30 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-import traceback
+
 from collections import defaultdict
 from functools import partial
 from itertools import combinations
+import traceback
+
 from picard import log
-from picard.metadata import Metadata
 from picard.dataobj import DataObject
-from picard.mbxml import media_formats_from_node, label_info_from_node
-from picard.util import uniqify
+from picard.mbjson import (
+    countries_from_node,
+    label_info_from_node,
+    media_formats_from_node,
+)
+from picard.metadata import Metadata
+from picard.util import (
+    limited_join,
+    uniqify,
+)
 
 
 class ReleaseGroup(DataObject):
 
-    def __init__(self, id):
-        DataObject.__init__(self, id)
+    def __init__(self, rg_id):
+        super().__init__(rg_id)
         self.metadata = Metadata()
         self.loaded = False
         self.versions = []
@@ -41,7 +57,7 @@ class ReleaseGroup(DataObject):
 
     def load_versions(self, callback):
         kwargs = {"release-group": self.id, "limit": 100}
-        self.tagger.xmlws.browse_releases(partial(self._request_finished, callback), **kwargs)
+        self.tagger.mb_api.browse_releases(partial(self._request_finished, callback), **kwargs)
 
     def _parse_versions(self, document):
         """Parse document and return a list of releases"""
@@ -58,44 +74,40 @@ class ReleaseGroup(DataObject):
             "catnum":   N_('Cat No'),
         }
         extrakeys = ("packaging", "barcode", "disambiguation")
-        for node in document.metadata[0].release_list[0].release:
-            labels, catnums = label_info_from_node(node.label_info_list[0])
 
-            countries = []
-            if 'release_event_list' in node.children:
-                for release_event in node.release_event_list[0].release_event:
-                    if "area" in release_event.children:
-                        countries.append(release_event.area[0].iso_3166_1_code_list[0].iso_3166_1_code[0].text)
+        try:
+            releases = document['releases']
+        except (TypeError, KeyError):
+            releases = []
 
+        max_tracks = 10
+        for node in releases:
+            labels, catnums = label_info_from_node(node['label-info'])
+
+            countries = countries_from_node(node)
+
+            if len(node['media']) > max_tracks:
+                tracks = "+".join(str(m['track-count']) for m in node['media'][:max_tracks]) + '+…'
+            else:
+                tracks = "+".join(str(m['track-count']) for m in node['media'])
             formats = []
-            for medium in node.medium_list[0].medium:
-                if "format" in medium.children:
-                    formats.append(medium.format[0].text)
+            for medium in node['media']:
+                if "format" in medium:
+                    formats.append(medium['format'])
             release = {
-                "id":      node.id,
-                "year":    node.date[0].text[:4] if "date" in node.children else "????",
-                "country": "+".join(countries) if countries
-                    else node.country[0].text if "country" in node.children
-                    else "??",
-                "format":  media_formats_from_node(node.medium_list[0]),
-                "label":  ", ".join([' '.join(x.split(' ')[:2]) for x in set(labels)]),
+                "id":      node['id'],
+                "year":    node['date'][:4] if "date" in node else "????",
+                "country": limited_join(countries, 10, '+', '…') if countries
+                else node.get('country', '') or "??",
+                "format":  media_formats_from_node(node['media']),
+                "label":  ", ".join(' '.join(x.split(' ')[:2]) for x in set(labels)),
                 "catnum": ", ".join(set(catnums)),
-                "tracks":  "+".join([m.track_list[0].count for m in node.medium_list[0].medium]),
-                "barcode":
-                    node.barcode[0].text
-                    if "barcode" in node.children
-                    and node.barcode[0].text != ""
-                    else _("[no barcode]"),
-                "packaging":
-                    node.packaging[0].text
-                    if "packaging" in node.children
-                    else None,
-                "disambiguation":
-                    node.disambiguation[0].text
-                    if "disambiguation" in node.children
-                    else None,
+                "tracks": tracks,
+                "barcode": node.get('barcode', '') or _('[no barcode]'),
+                "packaging": node.get('packaging', '') or '??',
+                "disambiguation": node.get('disambiguation', ''),
                 "_disambiguate_name": list(),
-                "totaltracks": sum([int(m.track_list[0].count) for m in node.medium_list[0].medium]),
+                "totaltracks": sum(m['track-count'] for m in node['media']),
                 "countries": countries,
                 "formats": formats,
             }
@@ -103,20 +115,20 @@ class ReleaseGroup(DataObject):
 
         versions = defaultdict(list)
         for release in data:
-            name = " / ".join([release[k] for k in namekeys]).replace("&", "&&")
+            name = " / ".join(release[k] for k in namekeys).replace("&", "&&")
             if name == release["tracks"]:
                 name = "%s / %s" % (_('[no release info]'), name)
             versions[name].append(release)
 
         # de-duplicate names if possible
-        for name, releases in versions.iteritems():
+        for name, releases in versions.items():
             for a, b in combinations(releases, 2):
                 for key in extrakeys:
                     (value1, value2) = (a[key], b[key])
                     if value1 != value2:
                         a['_disambiguate_name'].append(value1)
                         b['_disambiguate_name'].append(value2)
-        for name, releases in versions.iteritems():
+        for name, releases in versions.items():
             for release in releases:
                 dis = " / ".join(filter(None, uniqify(release['_disambiguate_name']))).replace("&", "&&")
                 disname = name if not dis else name + ' / ' + dis
@@ -133,19 +145,19 @@ class ReleaseGroup(DataObject):
     def _request_finished(self, callback, document, http, error):
         try:
             if error:
-                log.error("%r", unicode(http.errorString()))
+                log.error("%r", http.errorString())
             else:
                 try:
                     self._parse_versions(document)
-                except:
+                except BaseException:
                     error = True
                     log.error(traceback.format_exc())
         finally:
             self.loaded = True
             callback()
 
-    def remove_album(self, id):
-        self.loaded_albums.discard(id)
+    def remove_album(self, album_id):
+        self.loaded_albums.discard(album_id)
         self.refcount -= 1
         if self.refcount == 0:
             del self.tagger.release_groups[self.id]
