@@ -3,10 +3,10 @@
 # Picard, the next-generation MusicBrainz tagger
 #
 # Copyright (C) 2006-2007 Lukáš Lalinský
-# Copyright (C) 2009, 2018-2019 Philipp Wolfer
+# Copyright (C) 2009, 2018-2021 Philipp Wolfer
 # Copyright (C) 2011-2013 Michael Wiencek
 # Copyright (C) 2012 Chad Wilson
-# Copyright (C) 2013-2014, 2018 Laurent Monin
+# Copyright (C) 2013-2014, 2018, 2020-2021 Laurent Monin
 # Copyright (C) 2014 Sophist-UK
 # Copyright (C) 2016-2017 Sambhav Kothari
 # Copyright (C) 2018 Vishal Choudhary
@@ -32,9 +32,10 @@ from PyQt5 import (
     QtWidgets,
 )
 
-from picard import (
-    config,
-    log,
+from picard import log
+from picard.config import (
+    Option,
+    get_config,
 )
 from picard.mbjson import (
     artist_credit_from_node,
@@ -52,11 +53,10 @@ from picard.ui.ui_cdlookup import Ui_Dialog
 
 class CDLookupDialog(PicardDialog):
 
-    autorestore = False
     dialog_header_state = "cdlookupdialog_header_state"
 
     options = [
-        config.Option("persist", dialog_header_state, QtCore.QByteArray())
+        Option("persist", dialog_header_state, QtCore.QByteArray())
     ]
 
     def __init__(self, releases, disc, parent=None):
@@ -70,7 +70,8 @@ class CDLookupDialog(PicardDialog):
         release_list.setSortingEnabled(True)
         release_list.setAlternatingRowColors(True)
         release_list.setHeaderLabels([_("Album"), _("Artist"), _("Date"), _("Country"),
-                                      _("Labels"), _("Catalog #s"), _("Barcode")])
+                                      _("Labels"), _("Catalog #s"), _("Barcode"),
+                                      _("Disambiguation")])
         self.ui.submit_button.setIcon(QtGui.QIcon(":/images/cdrom.png"))
         if self.releases:
             def myjoin(values):
@@ -81,7 +82,7 @@ class CDLookupDialog(PicardDialog):
             for release in self.releases:
                 labels, catalog_numbers = label_info_from_node(release['label-info'])
                 dates, countries = release_dates_and_countries_from_node(release)
-                barcode = release['barcode'] if "barcode" in release else ""
+                barcode = release.get('barcode', '')
                 item = QtWidgets.QTreeWidgetItem(release_list)
                 if disc.mcn and compare_barcodes(barcode, disc.mcn):
                     selected = item
@@ -92,6 +93,7 @@ class CDLookupDialog(PicardDialog):
                 item.setText(4, myjoin(labels))
                 item.setText(5, myjoin(catalog_numbers))
                 item.setText(6, barcode)
+                item.setText(7, release.get('disambiguation', ''))
                 item.setData(0, QtCore.Qt.UserRole, release['id'])
             release_list.setCurrentItem(selected or release_list.topLevelItem(0))
             self.ui.ok_button.setEnabled(True)
@@ -104,7 +106,6 @@ class CDLookupDialog(PicardDialog):
             self.ui.results_view.setCurrentIndex(1)
         self.ui.lookup_button.clicked.connect(self.lookup)
         self.ui.submit_button.clicked.connect(self.lookup)
-        self.restore_geometry()
         self.restore_header_state()
         self.finished.connect(self.save_header_state)
 
@@ -124,6 +125,7 @@ class CDLookupDialog(PicardDialog):
     def restore_header_state(self):
         if self.ui.release_list:
             header = self.ui.release_list.header()
+            config = get_config()
             state = config.persist[self.dialog_header_state]
             if state:
                 header.restoreState(state)
@@ -132,5 +134,6 @@ class CDLookupDialog(PicardDialog):
     def save_header_state(self):
         if self.ui.release_list:
             state = self.ui.release_list.header().saveState()
+            config = get_config()
             config.persist[self.dialog_header_state] = state
             log.debug("save_state: %s" % self.dialog_header_state)
