@@ -6,9 +6,9 @@
 # Copyright (C) 2014 Sophist-UK
 # Copyright (C) 2016-2017 Sambhav Kothari
 # Copyright (C) 2017 Wieland Hoffmann
-# Copyright (C) 2017-2018 Laurent Monin
+# Copyright (C) 2017-2018, 2020-2021 Laurent Monin
 # Copyright (C) 2018 Vishal Choudhary
-# Copyright (C) 2019-2020 Philipp Wolfer
+# Copyright (C) 2019-2021 Philipp Wolfer
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -27,6 +27,7 @@
 
 from PyQt5 import (
     QtCore,
+    QtGui,
     QtWidgets,
 )
 
@@ -45,7 +46,7 @@ from picard.ui.ui_edittagdialog import Ui_EditTagDialog
 
 AUTOCOMPLETE_RELEASE_TYPES = [s.lower() for s
                               in sorted(RELEASE_PRIMARY_GROUPS) + sorted(RELEASE_SECONDARY_GROUPS)]
-AUTOCOMPLETE_RELEASE_STATUS = sorted([s.lower() for s in RELEASE_STATUS])
+AUTOCOMPLETE_RELEASE_STATUS = sorted(s.lower() for s in RELEASE_STATUS)
 AUTOCOMPLETE_RELEASE_COUNTRIES = sorted(RELEASE_COUNTRIES, key=str.casefold)
 AUTOCOMPLETE_RELEASE_FORMATS = sorted(RELEASE_FORMATS, key=str.casefold)
 
@@ -56,14 +57,14 @@ class TagEditorDelegate(QtWidgets.QItemDelegate):
         if not index.isValid():
             return None
         tag = self.get_tag_name(index)
-        if tag.partition(':')[0] in ('comment', 'lyrics'):
+        if tag.partition(':')[0] in {'comment', 'lyrics'}:
             editor = QtWidgets.QPlainTextEdit(parent)
             editor.setFrameStyle(editor.style().styleHint(QtWidgets.QStyle.SH_ItemView_DrawDelegateFrame, None, editor))
             editor.setMinimumSize(QtCore.QSize(0, 80))
         else:
             editor = super().createEditor(parent, option, index)
         completer = None
-        if tag in ('date', 'originaldate'):
+        if tag in {'date', 'originaldate'}:
             editor.setPlaceholderText(_('YYYY-MM-DD'))
         elif tag == 'originalyear':
             editor.setPlaceholderText(_('YYYY'))
@@ -90,8 +91,6 @@ class TagEditorDelegate(QtWidgets.QItemDelegate):
 
 class EditTagDialog(PicardDialog):
 
-    autorestore = False
-
     def __init__(self, window, tag):
         super().__init__(window)
         self.ui = Ui_EditTagDialog()
@@ -119,19 +118,46 @@ class EditTagDialog(PicardDialog):
         self.value_list.setItemDelegate(TagEditorDelegate(self))
         self.tag_changed(tag)
         self.value_selection_changed()
-        self.restore_geometry()
+
+    def keyPressEvent(self, event):
+        if event.modifiers() == QtCore.Qt.NoModifier and event.key() in {QtCore.Qt.Key_Enter, QtCore.Qt.Key_Return}:
+            self.add_or_edit_value()
+            event.accept()
+        elif event.matches(QtGui.QKeySequence.Delete):
+            self.remove_value()
+        elif event.key() == QtCore.Qt.Key_Insert:
+            self.add_value()
+        else:
+            super().keyPressEvent(event)
+
+    def tag_selected(self, index):
+        self.add_or_edit_value()
 
     def edit_value(self):
         item = self.value_list.currentItem()
         if item:
+            # Do not initialize editing if editor is already active. Avoids flickering of the edit field
+            # when already in edit mode. `isPersistentEditorOpen` is only supported in Qt 5.10 and later.
+            if hasattr(self.value_list, 'isPersistentEditorOpen') and self.value_list.isPersistentEditorOpen(item):
+                return
             self.value_list.editItem(item)
 
     def add_value(self):
         item = QtWidgets.QListWidgetItem()
         item.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsEditable)
         self.value_list.addItem(item)
+        self.value_list.setCurrentItem(item)
         self.value_list.editItem(item)
 
+    def add_or_edit_value(self):
+        last_item = self.value_list.item(self.value_list.count() - 1)
+        # Edit the last item, if it is empty, or add a new empty item
+        if last_item and not last_item.text():
+            self.value_list.setCurrentItem(last_item)
+            self.edit_value()
+        else:
+            self.add_value()
+
     def remove_value(self):
         value_list = self.value_list
         row = value_list.currentRow()
@@ -261,14 +287,12 @@ class EditTagDialog(PicardDialog):
                                              list(self.metadata_box.tag_diff.new[self.tag]) or [""])
 
     def accept(self):
-        self.window.ignore_selection_changes = True
-        for tag, values in self.modified_tags.items():
-            self.modified_tags[tag] = [v for v in values if v]
-        modified_tags = self.modified_tags.items()
-        for obj in self.metadata_box.objects:
-            for tag, values in modified_tags:
-                obj.metadata[tag] = list(values)
-            obj.update()
-        self.window.ignore_selection_changes = False
-        self.window.update_selection()
+        with self.window.ignore_selection_changes:
+            for tag, values in self.modified_tags.items():
+                self.modified_tags[tag] = [v for v in values if v]
+            modified_tags = self.modified_tags.items()
+            for obj in self.metadata_box.objects:
+                for tag, values in modified_tags:
+                    obj.metadata[tag] = list(values)
+                obj.update()
         super().accept()
