@@ -4,9 +4,9 @@
 #
 # Copyright (C) 2004 Robert Kaye
 # Copyright (C) 2006-2007 Lukáš Lalinský
-# Copyright (C) 2008, 2019 Philipp Wolfer
+# Copyright (C) 2008, 2019-2021 Philipp Wolfer
 # Copyright (C) 2012-2013 Michael Wiencek
-# Copyright (C) 2013, 2018-2019 Laurent Monin
+# Copyright (C) 2013, 2018-2021 Laurent Monin
 # Copyright (C) 2017 Sambhav Kothari
 #
 # This program is free software; you can redistribute it and/or
@@ -24,7 +24,10 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 
-from picard import config
+from picard.config import (
+    TextOption,
+    get_config,
+)
 from picard.util.cdrom import (
     AUTO_DETECT_DRIVES,
     DEFAULT_DRIVES,
@@ -50,9 +53,10 @@ class CDLookupOptionsPage(OptionsPage):
     PARENT = None
     SORT_ORDER = 50
     ACTIVE = True
+    HELP_URL = '/config/options_cdlookup.html'
 
     options = [
-        config.TextOption("setting", "cd_lookup_device", ",".join(DEFAULT_DRIVES)),
+        TextOption("setting", "cd_lookup_device", ",".join(DEFAULT_DRIVES)),
     ]
 
     def __init__(self, parent=None):
@@ -64,6 +68,7 @@ class CDLookupOptionsPage(OptionsPage):
             self.ui.cd_lookup_device.addItems(self._device_list)
 
     def load(self):
+        config = get_config()
         device = config.setting["cd_lookup_device"]
         if AUTO_DETECT_DRIVES:
             try:
@@ -74,6 +79,7 @@ class CDLookupOptionsPage(OptionsPage):
             self.ui.cd_lookup_device.setText(device)
 
     def save(self):
+        config = get_config()
         if AUTO_DETECT_DRIVES:
             device = self.ui.cd_lookup_device.currentText()
             device_list = self._device_list
