@@ -5,10 +5,10 @@
 # Copyright (C) 2006 Lukáš Lalinský
 # Copyright (C) 2011-2014 Michael Wiencek
 # Copyright (C) 2012 Frederik “Freso” S. Olesen
-# Copyright (C) 2013-2015, 2018-2019 Laurent Monin
+# Copyright (C) 2013-2015, 2018-2021 Laurent Monin
 # Copyright (C) 2016-2017 Sambhav Kothari
 # Copyright (C) 2017 Suhas
-# Copyright (C) 2018-2019 Philipp Wolfer
+# Copyright (C) 2018-2021 Philipp Wolfer
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -33,7 +33,10 @@ from PyQt5 import (
     QtWidgets,
 )
 
-from picard import config
+from picard.config import (
+    ListOption,
+    get_config,
+)
 from picard.const import (
     RELEASE_COUNTRIES,
     RELEASE_FORMATS,
@@ -158,9 +161,9 @@ class ReleasesOptionsPage(OptionsPage):
     HELP_URL = '/config/options_releases.html'
 
     options = [
-        config.ListOption("setting", "release_type_scores", _release_type_scores),
-        config.ListOption("setting", "preferred_release_countries", []),
-        config.ListOption("setting", "preferred_release_formats", []),
+        ListOption("setting", "release_type_scores", _release_type_scores),
+        ListOption("setting", "preferred_release_countries", []),
+        ListOption("setting", "preferred_release_formats", []),
     ]
 
     def __init__(self, parent=None):
@@ -172,11 +175,11 @@ class ReleasesOptionsPage(OptionsPage):
 
         def add_slider(name, griditer, context):
             label = pgettext_attributes(context, name)
-            self._release_type_sliders[name] = \
-                ReleaseTypeScore(self.ui.type_group,
-                                 self.ui.gridLayout,
-                                 label,
-                                 next(griditer))
+            self._release_type_sliders[name] = ReleaseTypeScore(
+                self.ui.type_group,
+                self.ui.gridLayout,
+                label,
+                next(griditer))
 
         griditer = RowColIter(len(RELEASE_PRIMARY_GROUPS)
                               + len(RELEASE_SECONDARY_GROUPS)
@@ -187,16 +190,25 @@ class ReleasesOptionsPage(OptionsPage):
                            key=lambda v: pgettext_attributes('release_group_secondary_type', v)):
             add_slider(name, griditer, context='release_group_secondary_type')
 
-        self.reset_preferred_types_btn = QtWidgets.QPushButton(self.ui.type_group)
-        self.reset_preferred_types_btn.setText(_("Reset all"))
+        reset_types_btn = QtWidgets.QPushButton(self.ui.type_group)
+        reset_types_btn.setText(_("Reset all"))
         sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
         sizePolicy.setHorizontalStretch(0)
         sizePolicy.setVerticalStretch(0)
-        sizePolicy.setHeightForWidth(self.reset_preferred_types_btn.sizePolicy().hasHeightForWidth())
-        self.reset_preferred_types_btn.setSizePolicy(sizePolicy)
+        sizePolicy.setHeightForWidth(reset_types_btn.sizePolicy().hasHeightForWidth())
+        reset_types_btn.setSizePolicy(sizePolicy)
         r, c = next(griditer)
-        self.ui.gridLayout.addWidget(self.reset_preferred_types_btn, r, c, 1, 2)
-        self.reset_preferred_types_btn.clicked.connect(self.reset_preferred_types)
+        self.ui.gridLayout.addWidget(reset_types_btn, r, c, 1, 2)
+        reset_types_btn.clicked.connect(self.reset_preferred_types)
+
+        self.setTabOrder(reset_types_btn, self.ui.country_list)
+        self.setTabOrder(self.ui.country_list, self.ui.preferred_country_list)
+        self.setTabOrder(self.ui.preferred_country_list, self.ui.add_countries)
+        self.setTabOrder(self.ui.add_countries, self.ui.remove_countries)
+        self.setTabOrder(self.ui.remove_countries, self.ui.format_list)
+        self.setTabOrder(self.ui.format_list, self.ui.preferred_format_list)
+        self.setTabOrder(self.ui.preferred_format_list, self.ui.add_formats)
+        self.setTabOrder(self.ui.add_formats, self.ui.remove_formats)
 
         self.ui.add_countries.clicked.connect(self.add_preferred_countries)
         self.ui.remove_countries.clicked.connect(self.remove_preferred_countries)
@@ -216,6 +228,7 @@ class ReleasesOptionsPage(OptionsPage):
         super().restore_defaults()
 
     def load(self):
+        config = get_config()
         scores = dict(config.setting["release_type_scores"])
         for (release_type, release_type_slider) in self._release_type_sliders.items():
             release_type_slider.setValue(scores.get(release_type,
@@ -227,6 +240,7 @@ class ReleasesOptionsPage(OptionsPage):
                               self.ui.format_list, self.ui.preferred_format_list)
 
     def save(self):
+        config = get_config()
         scores = []
         for (release_type, release_type_slider) in self._release_type_sliders.items():
             scores.append((release_type, release_type_slider.value()))
@@ -272,6 +286,7 @@ class ReleasesOptionsPage(OptionsPage):
         def fcmp(x):
             return strxfrm(x[1])
         source_list.sort(key=fcmp)
+        config = get_config()
         saved_data = config.setting[setting]
         move = []
         for data, name in source_list:
@@ -291,6 +306,7 @@ class ReleasesOptionsPage(OptionsPage):
         for i in range(list1.count()):
             item = list1.item(i)
             data.append(item.data(QtCore.Qt.UserRole))
+        config = get_config()
         config.setting[setting] = data
 
 
