@@ -3,12 +3,12 @@
 # Picard, the next-generation MusicBrainz tagger
 #
 # Copyright (C) 2007 Lukáš Lalinský
-# Copyright (C) 2013-2015, 2018-2019 Laurent Monin
+# Copyright (C) 2013-2015, 2018-2021 Laurent Monin
 # Copyright (C) 2016-2017 Sambhav Kothari
 # Copyright (C) 2017 Frederik “Freso” S. Olesen
 # Copyright (C) 2017 Sophist-UK
 # Copyright (C) 2018 Vishal Choudhary
-# Copyright (C) 2019 Philipp Wolfer
+# Copyright (C) 2019, 2021 Philipp Wolfer
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -30,7 +30,7 @@ from PyQt5 import (
     QtWidgets,
 )
 
-from picard import config
+from picard.config import get_config
 from picard.const.sys import (
     IS_MACOS,
     IS_WIN,
@@ -75,6 +75,7 @@ def enabledSlot(func, state):
 
 
 def find_starting_directory():
+    config = get_config()
     if config.setting["starting_directory"]:
         path = config.setting["starting_directory_path"]
     else:
