@@ -3,8 +3,8 @@
 #
 # Picard, the next-generation MusicBrainz tagger
 #
-# Copyright (C) 2020 Laurent Monin
-# Copyright (C) 2020 Philipp Wolfer
+# Copyright (C) 2020-2021 Laurent Monin
+# Copyright (C) 2020-2021 Philipp Wolfer
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -35,7 +35,11 @@ ALIASES = {
     'abhi-ohri': 'Abhinav Ohri',
     'Antonio Larrosa <alarrosa@suse.com>': 'Antonio Larrosa',
     'Lukas Lalinsky <lalinsky@gmail.com>': 'Lukáš Lalinský',
+    'petitminion': 'Petit Minion',
     'Philipp Wolfer <ph.wolfer@gmail.com>': 'Philipp Wolfer',
+    'Ray': 'Ray Bouchard',
+    'RaysDev': 'Ray Bouchard',
+    'Sophist': 'Sophist-UK',
     'vishal choudhary': 'Vishal Choudhary',
     'vishichoudhary': 'Vishal Choudhary',
     'yvanzo': 'Yvan Rivière',
@@ -109,13 +113,13 @@ def parse_copyright_text(text):
 EMPTY_LINE = ("\n", "#\n")
 
 
-def parse_file(path):
+def parse_file(path, encoding='utf-8'):
     authors_from_log = extract_authors_from_gitlog(path)
     start = end = None
     authors_from_file = {}
 
-    skip_pattern = re.compile(r'^(?:#|/\*|//)\s+(fix-header:\s*skip|Automatically\s+generated)', re.IGNORECASE)
-    with open(path) as f:
+    skip_pattern = re.compile(r'^(?:#|/\*|//)\s+(fix-header:\s*skip|Automatically\s+generated|Created\s+by:\s+The\s+Resource\s+Compiler\s+for\s+PyQt5)', re.IGNORECASE)
+    with open(path, encoding=encoding) as f:
         lines = f.readlines()
         found = defaultdict(lambda: None)
         if lines and lines[0].startswith('#!'):
@@ -207,8 +211,8 @@ LICENSE_BOTTOM = """#
 """
 
 
-def fix_header(path):
-    found, authors_from_file, authors_from_log, before, after = parse_file(path)
+def fix_header(path, encoding='utf-8'):
+    found, authors_from_file, authors_from_log, before, after = parse_file(path, encoding)
     if found['skip'] is not None:
         return None, found['skip']
 
@@ -260,6 +264,7 @@ def main():
     parser.add_argument('-e', '--extension', default='.py', help='File extension to filter by')
     parser.add_argument('-i', '--in-place', action='store_true', default=False, help='Edit files in place')
     parser.add_argument('-r', '--recursive', action='store_true', default=False, help='Search through subfolders')
+    parser.add_argument('--encoding', default='utf-8', help='File encoding of the source files')
     args = parser.parse_args()
 
     paths = list(args.path)
@@ -267,7 +272,7 @@ def main():
     for path in paths:
         if os.path.isfile(path):
             name, ext = os.path.splitext(path)
-            if args.extension in ('', ext):
+            if args.extension in {'', ext}:
                 files.add(path)
         else:
             if args.recursive:
@@ -277,13 +282,13 @@ def main():
         sys.exit(0)
 
     for path in files:
-        new_content, info = fix_header(path)
+        new_content, info = fix_header(path, encoding=args.encoding)
         if new_content is None:
             print("Skipping %s (%s)" % (path, info), file=sys.stderr)
             continue
         if args.in_place:
             print("Parsing and fixing %s (in place)" % path, file=sys.stderr)
-            with open(path, 'w') as f:
+            with open(path, 'w', encoding=args.encoding) as f:
                 print(new_content, file=f)
         else:
             # by default, we just output to stdout
