@@ -2,7 +2,8 @@
 #
 # Picard, the next-generation MusicBrainz tagger
 #
-# Copyright (C) 2019-2020 Philipp Wolfer
+# Copyright (C) 2019-2021 Philipp Wolfer
+# Copyright (C) 2020 Laurent Monin
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -75,6 +76,10 @@ class CommonCoverArtTests:
 
         def setUp(self):
             super().setUp()
+            self.set_config_values({
+                'clear_existing_tags': False,
+                'preserve_images': False,
+            })
             self.jpegdata = load_coverart_file('mb.jpg')
             self.pngdata = load_coverart_file('mb.png')
 
@@ -98,7 +103,7 @@ class CommonCoverArtTests:
         def test_cover_art_with_types(self):
             expected = set('abcdefg'[:]) if self.supports_types else set('a')
             loaded_metadata = save_and_load_metadata(self.filename, self._cover_metadata())
-            found = set([chr(img.data[-1]) for img in loaded_metadata.images])
+            found = {chr(img.data[-1]) for img in loaded_metadata.images}
             self.assertEqual(expected, found)
 
         @skipUnlessTestfile
@@ -118,6 +123,29 @@ class CommonCoverArtTests:
             loaded_metadata = save_and_load_metadata(self.filename, metadata)
             self.assertEqual(0, len(loaded_metadata.images))
 
+        @skipUnlessTestfile
+        def test_cover_art_clear_tags(self):
+            image = CoverArtImage(data=self.pngdata, types=['front'])
+            file_save_image(self.filename, image)
+            metadata = load_metadata(self.filename)
+            self.assertEqual(image, metadata.images[0])
+            config.setting['clear_existing_tags'] = True
+            config.setting['preserve_images'] = True
+            metadata = save_and_load_metadata(self.filename, Metadata())
+            self.assertEqual(image, metadata.images[0])
+            config.setting['preserve_images'] = False
+            metadata = save_and_load_metadata(self.filename, Metadata())
+            self.assertEqual(0, len(metadata.images))
+
+        @skipUnlessTestfile
+        def test_cover_art_clear_tags_preserve_images_no_existing_images(self):
+            config.setting['clear_existing_tags'] = True
+            config.setting['preserve_images'] = True
+            image = CoverArtImage(data=self.pngdata, types=['front'])
+            file_save_image(self.filename, image)
+            metadata = load_metadata(self.filename)
+            self.assertEqual(image, metadata.images[0])
+
         def _cover_metadata(self):
             imgdata = self.jpegdata
             metadata = Metadata()
