@@ -2,9 +2,11 @@
 #
 # Picard, the next-generation MusicBrainz tagger
 #
-# Copyright (C) 2019 Laurent Monin
-# Copyright (C) 2019-2020 Philipp Wolfer
 # Copyright (C) 2019 Zenara Daley
+# Copyright (C) 2019-2020 Laurent Monin
+# Copyright (C) 2019-2021 Philipp Wolfer
+# Copyright (C) 2020 raingloom
+# Copyright (C) 2021 Sophist-UK
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -28,7 +30,11 @@ import mutagen
 from test.picardtestcase import PicardTestCase
 
 from picard import config
-from picard.formats import id3
+from picard.file import File
+from picard.formats import (
+    id3,
+    open_,
+)
 from picard.metadata import Metadata
 
 from .common import (
@@ -162,6 +168,31 @@ class CommonId3Tests:
 
             self.assertEqual(len(new_metadata['performer:piano']), len(original_metadata['performer:piano']))
 
+        @skipUnlessTestfile
+        def test_performer_no_role_tmcl(self):
+            metadata = Metadata({
+                'performer': 'Performer',
+                'performer:piano': 'Performer Piano',
+            })
+            save_metadata(self.filename, metadata)
+            raw_metadata = load_raw(self.filename)
+            self.assertIn(['piano', 'Performer Piano'], raw_metadata['TMCL'].people)
+            self.assertIn(['performer', 'Performer'], raw_metadata['TMCL'].people)
+            self.assertNotIn('TXXX:performer', raw_metadata)
+
+        @skipUnlessTestfile
+        def test_performer_no_role_tipl(self):
+            config.setting['write_id3v23'] = True
+            metadata = Metadata({
+                'performer': 'Performer',
+                'performer:piano': 'Performer Piano',
+            })
+            save_metadata(self.filename, metadata)
+            raw_metadata = load_raw(self.filename)
+            self.assertIn(['piano', 'Performer Piano'], raw_metadata['TIPL'].people)
+            self.assertIn(['performer', 'Performer'], raw_metadata['TIPL'].people)
+            self.assertNotIn('TXXX:performer', raw_metadata)
+
         @skipUnlessTestfile
         def test_comment_delete(self):
             metadata = Metadata(self.tags)
@@ -454,6 +485,103 @@ class CommonId3Tests:
             self.assertIn('http://example.com/1', loaded_licenses)
             self.assertIn('http://example.com/2', loaded_licenses)
 
+        @skipUnlessTestfile
+        def test_license_upgrade_wcop(self):
+            tags = mutagen.id3.ID3Tags()
+            tags.add(mutagen.id3.WCOP(url='http://example.com/1'))
+            save_raw(self.filename, tags)
+            metadata = load_metadata(self.filename)
+            self.assertEqual('http://example.com/1', metadata['license'])
+            metadata.add('license', 'http://example.com/2')
+            save_metadata(self.filename, metadata)
+            raw_metadata = load_raw(self.filename)
+            self.assertNotIn('WCOP', raw_metadata)
+            loaded_licenses = [url for url in raw_metadata['TXXX:LICENSE']]
+            self.assertEqual(['http://example.com/1', 'http://example.com/2'], loaded_licenses)
+
+        @skipUnlessTestfile
+        def test_license_downgrade_wcop(self):
+            tags = mutagen.id3.ID3Tags()
+            licenses = ['http://example.com/1', 'http://example.com/2']
+            tags.add(mutagen.id3.TXXX(desc='LICENSE', text=licenses))
+            save_raw(self.filename, tags)
+            raw_metadata = load_raw(self.filename)
+            metadata = load_metadata(self.filename)
+            self.assertEqual(licenses, metadata.getall('license'))
+            metadata['license'] = 'http://example.com/1'
+            save_metadata(self.filename, metadata)
+            raw_metadata = load_raw(self.filename)
+            self.assertEqual('http://example.com/1', raw_metadata['WCOP'])
+            self.assertNotIn('TXXX:LICENSE', raw_metadata)
+
+        @skipUnlessTestfile
+        def test_license_delete(self):
+            tags = mutagen.id3.ID3Tags()
+            tags.add(mutagen.id3.WCOP(url='http://example.com/1'))
+            tags.add(mutagen.id3.TXXX(desc='LICENSE', text='http://example.com/2'))
+            save_raw(self.filename, tags)
+            metadata = load_metadata(self.filename)
+            del metadata['license']
+            loaded_metadata = save_and_load_metadata(self.filename, metadata)
+            self.assertNotIn('license', loaded_metadata)
+
+        @skipUnlessTestfile
+        def test_woar_not_duplicated(self):
+            metadata = Metadata({
+                'website': 'http://example.com/1'
+            })
+            loaded_metadata = save_and_load_metadata(self.filename, metadata)
+            self.assertEqual(metadata['website'], loaded_metadata['website'])
+            metadata['website'] = 'http://example.com/2'
+            loaded_metadata = save_and_load_metadata(self.filename, metadata)
+            self.assertEqual(metadata['website'], loaded_metadata['website'])
+
+        @skipUnlessTestfile
+        def test_woar_delete(self):
+            metadata = Metadata({
+                'website': 'http://example.com/1'
+            })
+            loaded_metadata = save_and_load_metadata(self.filename, metadata)
+            self.assertEqual(metadata['website'], loaded_metadata['website'])
+            del metadata['website']
+            loaded_metadata = save_and_load_metadata(self.filename, metadata)
+            self.assertNotIn('website', loaded_metadata)
+
+        @skipUnlessTestfile
+        def test_rating_email_non_latin1(self):
+            for rating in range(6):
+                config.setting['rating_user_email'] = 'foo€'
+                rating = '3'
+                metadata = Metadata({'~rating': rating})
+                loaded_metadata = save_and_load_metadata(self.filename, metadata)
+                self.assertEqual(loaded_metadata['~rating'], rating, '~rating: %r != %r' % (loaded_metadata['~rating'], rating))
+
+        @skipUnlessTestfile
+        def test_unchanged_metadata(self):
+            self.set_config_values({
+                'compare_ignore_tags': [],
+                'write_id3v23': True,
+            })
+            file = open_(self.filename)
+            file.orig_metadata = Metadata({
+                'album': 'somealbum',
+                'title': 'sometitle',
+                'date': '2021',
+                'originaldate': '2021',
+                'artists': 'foo/bar',
+            })
+            file.metadata = Metadata({
+                'album': 'somealbum',
+                'title': 'sometitle',
+                'date': '2021-12',
+                'originaldate': '2021-12-04',
+                'artists': ['foo', 'bar'],
+            })
+            file.state = File.NORMAL
+            file.update(signal=False)
+            self.assertEqual(file.similarity, 1.0)
+            self.assertEqual(file.state, File.NORMAL)
+
 
 class MP3Test(CommonId3Tests.Id3TestCase):
     testfile = 'test.mp3'
@@ -548,31 +676,56 @@ class AIFFTest(CommonId3Tests.Id3TestCase):
 
 
 class Id3UtilTest(PicardTestCase):
+    def test_id3encoding_from_config(self):
+        self.assertEqual(id3.Id3Encoding.LATIN1, id3.Id3Encoding.from_config('iso-8859-1'))
+        self.assertEqual(id3.Id3Encoding.UTF16, id3.Id3Encoding.from_config('utf-16'))
+        self.assertEqual(id3.Id3Encoding.UTF8, id3.Id3Encoding.from_config('utf-8'))
+
     def test_id3text(self):
         teststring = '日本語testÖäß'
-        self.assertEqual(id3.id3text(teststring, 0), '???testÖäß')
-        self.assertEqual(id3.id3text(teststring, 1), teststring)
-        self.assertEqual(id3.id3text(teststring, 3), teststring)
-
-    def test_image_type_from_id3_num(self):
-        self.assertEqual(id3.image_type_from_id3_num(0), 'other')
-        self.assertEqual(id3.image_type_from_id3_num(3), 'front')
-        self.assertEqual(id3.image_type_from_id3_num(6), 'medium')
-        self.assertEqual(id3.image_type_from_id3_num(9999), 'other')
-
-    def test_image_type_as_id3_num(self):
-        self.assertEqual(id3.image_type_as_id3_num('other'), 0)
-        self.assertEqual(id3.image_type_as_id3_num('front'), 3)
-        self.assertEqual(id3.image_type_as_id3_num('medium'), 6)
-        self.assertEqual(id3.image_type_as_id3_num('track'), 6)
-        self.assertEqual(id3.image_type_as_id3_num('unknowntype'), 0)
-
-    def test_types_from_id3(self):
-        self.assertEqual(id3.types_from_id3(0), ['other'])
-        self.assertEqual(id3.types_from_id3(3), ['front'])
-        self.assertEqual(id3.types_from_id3(6), ['medium'])
-        self.assertEqual(id3.types_from_id3(9999), ['other'])
+        self.assertEqual(id3.id3text(teststring, id3.Id3Encoding.LATIN1), '???testÖäß')
+        self.assertEqual(id3.id3text(teststring, id3.Id3Encoding.UTF16), teststring)
+        self.assertEqual(id3.id3text(teststring, id3.Id3Encoding.UTF16BE), teststring)
+        self.assertEqual(id3.id3text(teststring, id3.Id3Encoding.UTF8), teststring)
 
 
 class Mp3CoverArtTest(CommonCoverArtTests.CoverArtTestCase):
     testfile = 'test.mp3'
+
+
+class ID3FileTest(PicardTestCase):
+
+    def setUp(self):
+        super().setUp()
+        self.file = id3.ID3File('somepath/somefile.mp3')
+        config.setting['write_id3v23'] = False
+        config.setting['id3v23_join_with'] = ' / '
+        self.file.metadata['artist'] = ['foo', 'bar']
+        self.file.metadata['originaldate'] = '2020-04-01'
+        self.file.metadata['date'] = '2021-04-01'
+
+    def test_format_specific_metadata_v24(self):
+        metadata = self.file.metadata
+        for name, values in metadata.rawitems():
+            self.assertEqual(values, self.file.format_specific_metadata(metadata, name))
+
+    def test_format_specific_metadata_v23(self):
+        config.setting['write_id3v23'] = True
+        metadata = self.file.metadata
+        self.assertEqual(['foo / bar'], self.file.format_specific_metadata(metadata, 'artist'))
+        self.assertEqual(['2020'], self.file.format_specific_metadata(metadata, 'originaldate'))
+        self.assertEqual(['2021-04-01'], self.file.format_specific_metadata(metadata, 'date'))
+
+    def test_format_specific_metadata_v23_incomplete_date(self):
+        config.setting['write_id3v23'] = True
+        metadata = self.file.metadata
+        metadata['date'] = '2021-04'
+        self.assertEqual(['2021'], self.file.format_specific_metadata(metadata, 'date'))
+
+    def test_format_specific_metadata_override_settings(self):
+        settings = {
+            'write_id3v23': True,
+            'id3v23_join_with': '; ',
+        }
+        metadata = self.file.metadata
+        self.assertEqual(['foo; bar'], self.file.format_specific_metadata(metadata, 'artist', settings))
