@@ -2,7 +2,8 @@
 #
 # Picard, the next-generation MusicBrainz tagger
 #
-# Copyright (C) 2019 Philipp Wolfer
+# Copyright (C) 2019-2021 Philipp Wolfer
+# Copyright (C) 2020 Laurent Monin
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -19,12 +20,22 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 
-from .common import CommonTests
+from picard.formats import midi
+
+from .common import (
+    TAGS,
+    CommonTests,
+)
 
 
 class MIDITest(CommonTests.SimpleFormatsTestCase):
     testfile = 'test.mid'
     expected_info = {
         'length': 127997,
+        '~format': 'Standard MIDI File'
     }
     unexpected_info = ['~video']
+
+    def test_supports_tag(self):
+        for tag in TAGS:
+            self.assertFalse(midi.MIDIFile.supports_tag(tag))
