@@ -5,7 +5,10 @@
 # Copyright (C) 2017 Sambhav Kothari
 # Copyright (C) 2018 Wieland Hoffmann
 # Copyright (C) 2019-2020 Philipp Wolfer
+# Copyright (C) 2020 Laurent Monin
 # Copyright (C) 2020 Ray Bouchard
+# Copyright (C) 2021 Bob Swift
+# Copyright (C) 2021 Vladislav Karbovskii
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -27,7 +30,6 @@ import os
 
 from test.picardtestcase import PicardTestCase
 
-from picard import config
 from picard.acoustid.json_helpers import parse_recording
 from picard.mbjson import recording_to_metadata
 from picard.metadata import Metadata
@@ -39,7 +41,8 @@ settings = {
     "standardize_artists": False,
     "standardize_releases": False,
     "translate_artist_names": True,
-    "artist_locale": 'en'
+    "artist_locales": ['en'],
+    "translate_artist_names_script_exception": False,
 }
 
 
@@ -49,7 +52,7 @@ class AcoustIDTest(PicardTestCase):
         self.init_test(self.filename)
 
     def init_test(self, filename):
-        config.setting = settings.copy()
+        self.set_config_values(settings)
         self.json_doc = None
         with open(os.path.join('test', 'data', 'ws_data', filename), encoding='utf-8') as f:
             self.json_doc = json.load(f)
