@@ -2,8 +2,8 @@
 #
 # Picard, the next-generation MusicBrainz tagger
 #
-# Copyright (C) 2019 Philipp Wolfer
 # Copyright (C) 2019-2020 Laurent Monin
+# Copyright (C) 2019-2021 Philipp Wolfer
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -74,6 +74,24 @@ class TestPicardConfig(TestPicardConfigCommon):
         self.assertEqual(self.config.setting["text_option"], "abc")
 
 
+class TestPicardConfigSection(TestPicardConfigCommon):
+
+    def test_as_dict(self):
+        TextOption("setting", "text_option", "abc")
+        BoolOption("setting", "bool_option", True)
+        IntOption("setting", "int_option", 42)
+
+        self.config.setting["int_option"] = 123
+
+        expected = {
+            "text_option": "abc",
+            "bool_option": True,
+            "int_option": 123,
+        }
+
+        self.assertEqual(expected, self.config.setting.as_dict())
+
+
 class TestPicardConfigTextOption(TestPicardConfigCommon):
 
     # TextOption
@@ -337,37 +355,37 @@ class TestPicardConfigVarOption(TestPicardConfigCommon):
         self.assertEqual(opt.convert(["a", "b", "a"]), {"a", "b"})
 
     def test_var_opt_no_config(self):
-        Option("setting", "var_option", set(["a", "b"]))
+        Option("setting", "var_option", {"a", "b"})
 
         # test default, nothing in config yet
-        self.assertEqual(self.config.setting["var_option"], set(["a", "b"]))
+        self.assertEqual(self.config.setting["var_option"], {"a", "b"})
         self.assertIs(type(self.config.setting["var_option"]), set)
 
     def test_var_opt_set_read_back(self):
-        Option("setting", "var_option", set(["a", "b"]))
+        Option("setting", "var_option", {"a", "b"})
 
         # set option to "def", and read back
-        self.config.setting["var_option"] = set(["c", "d"])
-        self.assertEqual(self.config.setting["var_option"], set(["c", "d"]))
+        self.config.setting["var_option"] = {"c", "d"}
+        self.assertEqual(self.config.setting["var_option"], {"c", "d"})
         self.assertIs(type(self.config.setting["var_option"]), set)
 
     def test_var_opt_set_none(self):
-        Option("setting", "var_option", set(["a", "b"]))
+        Option("setting", "var_option", {"a", "b"})
 
         # set option to None
         self.config.setting["var_option"] = None
-        self.assertEqual(self.config.setting["var_option"], set(["a", "b"]))
+        self.assertEqual(self.config.setting["var_option"], {"a", "b"})
 
     def test_var_opt_set_empty(self):
-        Option("setting", "var_option", set(["a", "b"]))
+        Option("setting", "var_option", {"a", "b"})
 
         # set option to ""
         self.config.setting["var_option"] = set()
         self.assertEqual(self.config.setting["var_option"], set())
 
     def test_var_opt_invalid_value(self):
-        Option("setting", "var_option", set(["a", "b"]))
+        Option("setting", "var_option", {"a", "b"})
 
         # store invalid value in config file directly
         self.config.setValue('setting/var_option', object)
-        self.assertEqual(self.config.setting["var_option"], set(["a", "b"]))
+        self.assertEqual(self.config.setting["var_option"], {"a", "b"})
