@@ -3,9 +3,12 @@
 # Picard, the next-generation MusicBrainz tagger
 #
 # Copyright (C) 2017 Sambhav Kothari
-# Copyright (C) 2017, 2019 Laurent Monin
+# Copyright (C) 2017, 2019-2021 Laurent Monin
 # Copyright (C) 2018 Wieland Hoffmann
-# Copyright (C) 2018-2020 Philipp Wolfer
+# Copyright (C) 2018-2021 Philipp Wolfer
+# Copyright (C) 2020 dukeyin
+# Copyright (C) 2021 Bob Swift
+# Copyright (C) 2021 Vladislav Karbovskii
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -30,6 +33,8 @@ from test.picardtestcase import (
 from picard import config
 from picard.album import Album
 from picard.mbjson import (
+    _node_skip_empty_iter,
+    _translate_artist_node,
     artist_to_metadata,
     countries_from_node,
     get_score,
@@ -37,6 +42,7 @@ from picard.mbjson import (
     media_formats_from_node,
     medium_to_metadata,
     recording_to_metadata,
+    release_dates_and_countries_from_node,
     release_group_to_metadata,
     release_to_metadata,
     track_to_metadata,
@@ -51,20 +57,40 @@ settings = {
     "standardize_artists": False,
     "standardize_releases": False,
     "translate_artist_names": True,
+    "translate_artist_names_script_exception": False,
     "standardize_instruments": True,
     "release_ars": True,
     "preferred_release_countries": [],
-    "artist_locale": 'en'
+    "artist_locales": ['en'],
 }
 
 
+class MBJSONItersTest(PicardTestCase):
+    def test_node_skip_empty_iter(self):
+        d = {
+            'bool_false': False,
+            'bool_true': True,
+            'int_0': 0,
+            'int_1': 1,
+            'float_0': 0.0,
+            'float_1': 1.1,
+            'list_empty': [],
+            'list_non_empty': ['a'],
+            'dict_empty': {},
+            'dict_non_empty': {'a': 'b'},
+        }
+        expected = set(d) - {'list_empty', 'dict_empty'}
+        result = set({k: v for k, v in _node_skip_empty_iter(d)})
+        self.assertSetEqual(expected, result)
+
+
 class MBJSONTest(PicardTestCase):
     def setUp(self):
         super().setUp()
         self.init_test(self.filename)
 
     def init_test(self, filename):
-        config.setting = settings.copy()
+        self.set_config_values(settings)
         self.json_doc = load_test_json(filename)
 
 
@@ -92,6 +118,8 @@ class ReleaseTest(MBJSONTest):
         self.assertEqual(m['script'], 'Latn')
         self.assertEqual(m['~albumartists'], 'Pink Floyd')
         self.assertEqual(m['~albumartists_sort'], 'Pink Floyd')
+        self.assertEqual(m['~releasecomment'], 'stereo')
+        self.assertEqual(m['~releaseannotation'], 'Original Vinyl release')
         self.assertEqual(m['~releaselanguage'], 'eng')
         self.assertEqual(m.getall('~releasecountries'), ['GB', 'NZ'])
         self.assertEqual(a.genres, {
@@ -186,10 +214,15 @@ class RecordingTest(MBJSONTest):
         self.assertEqual(m['performer:guitar family'], 'Ed Sheeran')
         self.assertEqual(m['title'], 'Thinking Out Loud')
         self.assertEqual(m['work'], 'Thinking Out Loud')
+        self.assertEqual(m['~workcomment'], 'Ed Sheeran song')
         self.assertEqual(m['writer'], 'Ed Sheeran; Amy Wadge')
         self.assertEqual(m['~artists_sort'], 'Sheeran, Ed')
         self.assertEqual(m['~length'], '4:41')
         self.assertEqual(m['~recordingtitle'], 'Thinking Out Loud')
+        self.assertEqual(m['~recording_firstreleasedate'], '2014-06-20')
+        self.assertEqual(m['~video'], '')
+        self.assertNotIn('originaldate', m)
+        self.assertNotIn('originalyear', m)
         self.assertEqual(t.genres, {
             'blue-eyed soul': 1,
             'pop': 3})
@@ -207,6 +240,37 @@ class RecordingTest(MBJSONTest):
         self.assertEqual(m['performer:acoustic guitar'], 'Ed Sheeran')
 
 
+class RecordingComposerCreditsTest(MBJSONTest):
+
+    filename = 'recording_composer.json'
+
+    def test_standardize_artists(self):
+        m = Metadata()
+        t = Track('1')
+        config.setting['translate_artist_names'] = False
+        config.setting['standardize_artists'] = True
+        recording_to_metadata(self.json_doc, m, t)
+        self.assertEqual(m['composer'], 'Пётр Ильич Чайковский')
+        self.assertEqual(m['composersort'], 'Tchaikovsky, Pyotr Ilyich')
+
+    def test_use_credited_as(self):
+        m = Metadata()
+        t = Track('1')
+        config.setting['translate_artist_names'] = False
+        config.setting['standardize_artists'] = False
+        recording_to_metadata(self.json_doc, m, t)
+        self.assertEqual(m['composer'], 'Tchaikovsky')
+        self.assertEqual(m['composersort'], 'Tchaikovsky, Pyotr Ilyich')
+
+    def test_translate(self):
+        m = Metadata()
+        t = Track('1')
+        config.setting['translate_artist_names'] = True
+        recording_to_metadata(self.json_doc, m, t)
+        self.assertEqual(m['composer'], 'Pyotr Ilyich Tchaikovsky')
+        self.assertEqual(m['composersort'], 'Tchaikovsky, Pyotr Ilyich')
+
+
 class RecordingInstrumentalTest(MBJSONTest):
 
     filename = 'recording_instrumental.json'
@@ -220,6 +284,19 @@ class RecordingInstrumentalTest(MBJSONTest):
         self.assertNotIn('lyricist', m)
 
 
+class RecordingVideoTest(MBJSONTest):
+
+    filename = 'recording_video.json'
+
+    def test_recording(self):
+        m = Metadata()
+        t = Track('1')
+        recording_to_metadata(self.json_doc, m, t)
+        self.assertEqual(m['director'], 'Edward 209')
+        self.assertEqual(m['producer'], 'Edward 209')
+        self.assertEqual(m['~video'], '1')
+
+
 class NullRecordingTest(MBJSONTest):
 
     filename = 'recording_null.json'
@@ -348,6 +425,75 @@ class NullArtistTest(MBJSONTest):
         self.assertEqual(m, {})
 
 
+class ArtistTranslationTest(MBJSONTest):
+
+    filename = 'artist.json'
+
+    def test_locale_specific_match_first(self):
+        settings = {
+            "standardize_tracks": False,
+            "standardize_artists": False,
+            "standardize_releases": False,
+            "translate_artist_names": True,
+            "translate_artist_names_script_exception": False,
+            "standardize_instruments": True,
+            "release_ars": True,
+            "preferred_release_countries": [],
+            "artist_locales": ['en_CA', 'en'],
+        }
+        self.set_config_values(settings)
+        (artist_name, artist_sort_name) = _translate_artist_node(self.json_doc)
+        self.assertEqual(artist_name, 'Ed Sheeran (en_CA)')
+
+    def test_locale_specific_match_second(self):
+        settings = {
+            "standardize_tracks": False,
+            "standardize_artists": False,
+            "standardize_releases": False,
+            "translate_artist_names": True,
+            "translate_artist_names_script_exception": False,
+            "standardize_instruments": True,
+            "release_ars": True,
+            "preferred_release_countries": [],
+            "artist_locales": ['en_UK', 'en'],
+        }
+        self.set_config_values(settings)
+        (artist_name, artist_sort_name) = _translate_artist_node(self.json_doc)
+        self.assertEqual(artist_name, 'Ed Sheeran (en)')
+
+    def test_artist_match_root_locale_fallback(self):
+        settings = {
+            "standardize_tracks": False,
+            "standardize_artists": False,
+            "standardize_releases": False,
+            "translate_artist_names": True,
+            "translate_artist_names_script_exception": False,
+            "standardize_instruments": True,
+            "release_ars": True,
+            "preferred_release_countries": [],
+            "artist_locales": ['en_UK'],
+        }
+        self.set_config_values(settings)
+        (artist_name, artist_sort_name) = _translate_artist_node(self.json_doc)
+        self.assertEqual(artist_name, 'Ed Sheeran (en)')
+
+    def test_artist_no_match(self):
+        settings = {
+            "standardize_tracks": False,
+            "standardize_artists": False,
+            "standardize_releases": False,
+            "translate_artist_names": True,
+            "translate_artist_names_script_exception": False,
+            "standardize_instruments": True,
+            "release_ars": True,
+            "preferred_release_countries": [],
+            "artist_locales": ['de'],
+        }
+        self.set_config_values(settings)
+        (artist_name, artist_sort_name) = _translate_artist_node(self.json_doc)
+        self.assertEqual(artist_name, 'Ed Sheeran')
+
+
 class ReleaseGroupTest(MBJSONTest):
 
     filename = 'release_group.json'
@@ -357,6 +503,7 @@ class ReleaseGroupTest(MBJSONTest):
         r = ReleaseGroup("1")
         release_group_to_metadata(self.json_doc, m, r)
         self.assertEqual(m['musicbrainz_releasegroupid'], 'f5093c06-23e3-404f-aeaa-40f72885ee3a')
+        self.assertEqual(m['~releasegroup_firstreleasedate'], '1973-03-24')
         self.assertEqual(m['originaldate'], '1973-03-24')
         self.assertEqual(m['originalyear'], '1973')
         self.assertEqual(m['releasetype'], 'album')
@@ -389,6 +536,11 @@ class CountriesFromNodeTest(MBJSONTest):
         countries = countries_from_node(self.json_doc)
         self.assertEqual([], countries)
 
+    def test_countries_from_node_no_area(self):
+        del self.json_doc["release-events"][0]["area"]
+        countries = countries_from_node(self.json_doc)
+        self.assertEqual([], countries)
+
 
 class CountriesFromNodeNullTest(MBJSONTest):
 
@@ -399,6 +551,32 @@ class CountriesFromNodeNullTest(MBJSONTest):
         self.assertEqual(countries, [])
 
 
+class DatesCountriesFromNodeTest(MBJSONTest):
+
+    filename = 'country.json'
+
+    def test_dates_countries_from_node(self):
+        dates, countries = release_dates_and_countries_from_node(self.json_doc)
+        self.assertEqual(['GB'], countries)
+        self.assertEqual(['1986-03'], dates)
+
+    def test_dates_countries_from_node_no_event(self):
+        del self.json_doc["release-events"]
+        dates, countries = release_dates_and_countries_from_node(self.json_doc)
+        self.assertEqual([], countries)
+        self.assertEqual([], dates)
+
+
+class DatesCountriesFromNodeNullTest(MBJSONTest):
+
+    filename = 'country_null.json'
+
+    def test_dates_countries_from_node(self):
+        dates, countries = release_dates_and_countries_from_node(self.json_doc)
+        self.assertEqual(countries, [])
+        self.assertEqual([''], dates)
+
+
 class LabelInfoTest(MBJSONTest):
 
     filename = 'label_info.json'
