@@ -3,9 +3,10 @@
 # Picard, the next-generation MusicBrainz tagger
 #
 # Copyright (C) 2017 Sophist-UK
-# Copyright (C) 2018 Wieland Hoffmann
+# Copyright (C) 2018, 2020 Wieland Hoffmann
 # Copyright (C) 2018-2020 Laurent Monin
 # Copyright (C) 2018-2020 Philipp Wolfer
+# Copyright (C) 2020 dukeyin
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -29,7 +30,6 @@ from test.picardtestcase import (
 )
 from test.test_coverart_image import create_image
 
-from picard import config
 from picard.cluster import Cluster
 from picard.coverart.image import CoverArtImage
 from picard.file import File
@@ -74,7 +74,7 @@ class CommonTests:
 
         def setUp(self):
             super().setUp()
-            config.setting = settings.copy()
+            self.set_config_values(settings)
             self.metadata = self.get_metadata_object()
             self.metadata.length = 242
             self.metadata["single1"] = "single1-value"
