@@ -3,16 +3,17 @@
 # Picard, the next-generation MusicBrainz tagger
 #
 # Copyright (C) 2007 Lukáš Lalinský
-# Copyright (C) 2010, 2014, 2018-2020 Philipp Wolfer
+# Copyright (C) 2010, 2014, 2018-2021 Philipp Wolfer
 # Copyright (C) 2012 Chad Wilson
 # Copyright (C) 2013 Michael Wiencek
 # Copyright (C) 2013, 2017-2020 Laurent Monin
 # Copyright (C) 2014, 2017 Sophist-UK
 # Copyright (C) 2016-2017 Sambhav Kothari
 # Copyright (C) 2017 Antonio Larrosa
-# Copyright (C) 2017-2018 Wieland Hoffmann
+# Copyright (C) 2017-2018, 2021 Wieland Hoffmann
 # Copyright (C) 2018 virusMac
-# Copyright (C) 2020 Bob Swift
+# Copyright (C) 2020-2021 Bob Swift
+# Copyright (C) 2021 Adam James
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -29,15 +30,16 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 
+import builtins
 import copy
 import datetime
 import re
 import unittest
+from unittest import mock
 from unittest.mock import MagicMock
 
 from test.picardtestcase import PicardTestCase
 
-from picard import config
 from picard.cluster import Cluster
 from picard.const import DEFAULT_FILE_NAMING_FORMAT
 from picard.metadata import (
@@ -55,6 +57,7 @@ from picard.script import (
     ScriptParser,
     ScriptRuntimeError,
     ScriptSyntaxError,
+    ScriptUnicodeError,
     ScriptUnknownFunction,
     register_script_function,
     script_function,
@@ -106,9 +109,9 @@ class ScriptParserTest(PicardTestCase):
 
     def setUp(self):
         super().setUp()
-        config.setting = {
+        self.set_config_values({
             'enabled_plugins': '',
-        }
+        })
 
         self.parser = ScriptParser()
 
@@ -153,6 +156,23 @@ class ScriptParserTest(PicardTestCase):
 
         self.assertRegex(repr(item), regex)
 
+    def test_script_unicode_char(self):
+        self.assertScriptResultEquals("\\u6e56", "湖")
+        self.assertScriptResultEquals("foo\\u6e56bar", "foo湖bar")
+        self.assertScriptResultEquals("\\uFFFF", "\uffff")
+
+    def test_script_unicode_char_eof(self):
+        areg = r"^\d+:\d+: Unexpected end of script"
+        with self.assertRaisesRegex(ScriptEndOfFile, areg):
+            self.parser.eval("\\u")
+        with self.assertRaisesRegex(ScriptEndOfFile, areg):
+            self.parser.eval("\\uaff")
+
+    def test_script_unicode_char_err(self):
+        areg = r"^\d+:\d+: Invalid unicode character '\\ufffg'"
+        with self.assertRaisesRegex(ScriptUnicodeError, areg):
+            self.parser.eval("\\ufffg")
+
     def test_script_function_decorator_default(self):
         # test default decorator and default prefix
         @script_function()
@@ -530,6 +550,23 @@ class ScriptParserTest(PicardTestCase):
     def test_cmd_replace(self):
         self.assertScriptResultEquals("$replace(abc ab abd a,ab,test)", "testc test testd a")
 
+    def test_cmd_replacemulti(self):
+        context = Metadata()
+        context["genre"] = ["Electronic", "Idm", "Techno"]
+        self.assertScriptResultEquals("$replacemulti(%genre%,Idm,IDM)", "Electronic; IDM; Techno", context)
+
+        context["genre"] = ["Electronic", "Jungle", "Bardcore"]
+        self.assertScriptResultEquals("$replacemulti(%genre%,Bardcore,Hardcore)", "Electronic; Jungle; Hardcore", context)
+
+        context["test"] = ["One", "Two", "Three"]
+        self.assertScriptResultEquals("$replacemulti(%test%,Four,Five)", "One; Two; Three", context)
+
+        context["test"] = ["Four", "Five", "Six"]
+        self.assertScriptResultEquals("$replacemulti(%test%,Five,)", "Four; Six", context)
+
+        self.assertScriptResultEquals("$replacemulti(a; b,,,)", "a; b")
+        self.assertScriptResultEquals("$setmulti(foo,a; b)$replacemulti(%foo%,,,)", "a; b")
+
     def test_cmd_strip(self):
         self.assertScriptResultEquals("$strip(  \t abc  de \n f  )", "abc de f")
 
@@ -896,13 +933,40 @@ class ScriptParserTest(PicardTestCase):
     def test_cmd_performer(self):
         context = Metadata()
         context['performer:guitar'] = 'Foo1'
-        context['performer:rhythm-guitar'] = 'Foo2'
+        context['performer:rhythm-guitar'] = ['Foo2', 'Foo3']
         context['performer:drums'] = 'Drummer'
+        # Matches pattern
         result = self.parser.eval("$performer(guitar)", context=context)
-        performers = result.split(', ')
-        self.assertIn('Foo1', performers)
-        self.assertIn('Foo2', performers)
-        self.assertEqual(2, len(performers))
+        self.assertEqual({'Foo1', 'Foo2', 'Foo3'}, set(result.split(', ')))
+        # Empty pattern returns all performers
+        result = self.parser.eval("$performer()", context=context)
+        self.assertEqual({'Foo1', 'Foo2', 'Foo3', 'Drummer'}, set(result.split(', ')))
+        self.assertScriptResultEquals("$performer(perf)", "", context)
+
+    def test_cmd_performer_regex(self):
+        context = Metadata()
+        context['performer:guitar'] = 'Foo1'
+        context['performer:guitars'] = 'Foo2'
+        context['performer:rhythm-guitar'] = 'Foo3'
+        context['performer:drums (drum kit)'] = 'Drummer'
+        result = self.parser.eval(r"$performer(/^guitar/)", context=context)
+        self.assertEqual({'Foo1', 'Foo2'}, set(result.split(', ')))
+        result = self.parser.eval(r"$performer(/^guitar\$/)", context=context)
+        self.assertEqual({'Foo1'}, set(result.split(', ')))
+
+    def test_cmd_performer_regex_invalid(self):
+        context = Metadata()
+        context['performer:drums (drum kit)'] = 'Drummer'
+        self.assertScriptResultEquals(r"$performer(/drums \(/)", "", context)
+        self.assertScriptResultEquals(r"$performer(drums \()", "Drummer", context)
+
+    def test_cmd_performer_regex_ignore_case(self):
+        context = Metadata()
+        context['performer:guitar'] = 'Foo1'
+        context['performer:GUITARS'] = 'Foo2'
+        context['performer:rhythm-guitar'] = 'Foo3'
+        result = self.parser.eval(r"$performer(/^guitars?/i)", context=context)
+        self.assertEqual({'Foo1', 'Foo2'}, set(result.split(', ')))
 
     def test_cmd_performer_custom_join(self):
         context = Metadata()
@@ -910,10 +974,13 @@ class ScriptParserTest(PicardTestCase):
         context['performer:rhythm-guitar'] = 'Foo2'
         context['performer:drums'] = 'Drummer'
         result = self.parser.eval("$performer(guitar, / )", context=context)
-        performers = result.split(' / ')
-        self.assertIn('Foo1', performers)
-        self.assertIn('Foo2', performers)
-        self.assertEqual(2, len(performers))
+        self.assertEqual({'Foo1', 'Foo2'}, set(result.split(' / ')))
+
+    def test_cmd_performer_multi_colons(self):
+        context = Metadata()
+        context['performer:CV:松井栞里'] = '仁奈(CV:大出千夏)'
+        result = self.parser.eval("$performer(CV:松井栞里)", context=context)
+        self.assertEqual('仁奈(CV:大出千夏)', result)
 
     def test_cmd_matchedtracks(self):
         file = MagicMock()
@@ -976,6 +1043,12 @@ class ScriptParserTest(PicardTestCase):
     def test_char_escape(self):
         self.assertScriptResultEquals(r"\n\t\$\%\(\)\,\\", "\n\t$%(),\\")
 
+    def test_char_escape_unexpected_char(self):
+        self.assertRaises(ScriptSyntaxError, self.parser.eval, r'\x')
+
+    def test_char_escape_end_of_file(self):
+        self.assertRaises(ScriptEndOfFile, self.parser.eval, 'foo\\')
+
     def test_raise_unknown_function(self):
         self.assertRaises(ScriptUnknownFunction, self.parser.eval, '$unknownfn()')
 
@@ -1188,19 +1261,28 @@ class ScriptParserTest(PicardTestCase):
 
     def test_cmd_map(self):
         context = Metadata()
-        context["foo"] = "First:A; Second:B; Third:C"
-        context["bar"] = ["First:A", "Second:B", "Third:C"]
         foo_output = "1=FIRST:A; SECOND:B; THIRD:C"
         loop_output = "1=FIRST:A; 2=SECOND:B; 3=THIRD:C"
         alternate_output = "1=FIRST:2=A; SECOND:3=B; THIRD:4=C"
         # Tests with context
+        context["foo"] = "First:A; Second:B; Third:C"
         self.assertScriptResultEquals("$map(%foo%,$upper(%_loop_count%=%_loop_value%))", foo_output, context)
+        context["bar"] = ["First:A", "Second:B", "Third:C"]
         self.assertScriptResultEquals("$map(%bar%,$upper(%_loop_count%=%_loop_value%))", loop_output, context)
         # Tests with static inputs
         self.assertScriptResultEquals("$map(First:A; Second:B; Third:C,$upper(%_loop_count%=%_loop_value%))", loop_output, context)
+        # Tests for removing empty elements
+        context["baz"] = ["First:A", "Second:B", "Remove", "Third:C"]
+        test_output = "1=FIRST:A; 2=SECOND:B; 4=THIRD:C"
+        self.assertScriptResultEquals("$lenmulti(%baz%)", "4", context)
+        self.assertScriptResultEquals("$map(%baz%,$if($eq(%_loop_count%,3),,$upper(%_loop_count%=%_loop_value%)))", test_output, context)
+        context["baz"] = ["First:A", "Second:B", "Remove", "Third:C"]
+        self.assertScriptResultEquals("$setmulti(baz,$map(%baz%,$if($eq(%_loop_count%,3),,$upper(%_loop_count%=%_loop_value%))))%baz%", test_output, context)
+        self.assertScriptResultEquals("$lenmulti(%baz%)", "3", context)
+        self.assertScriptResultEquals("%baz%", test_output, context)
         # Tests with missing inputs
         self.assertScriptResultEquals("$map(,$upper(%_loop_count%=%_loop_value%))", "", context)
-        self.assertScriptResultEquals("$map(First:A; Second:B; Third:C,)", "; ; ", context)
+        self.assertScriptResultEquals("$map(First:A; Second:B; Third:C,)", "", context)
         # Tests with separator override
         self.assertScriptResultEquals("$map(First:A; Second:B; Third:C,$upper(%_loop_count%=%_loop_value%),:)", alternate_output, context)
         # Tests with invalid number of arguments
@@ -1272,6 +1354,7 @@ class ScriptParserTest(PicardTestCase):
         self.assertScriptResultEquals("$slice(First:A; Second:B; Third:C,-2,2)", output_1_2, context)
         # Tests with missing inputs
         self.assertScriptResultEquals("$slice(First:A; Second:B; Third:C,,1)", output_0_1, context)
+        self.assertScriptResultEquals("$slice(First:A; Second:B; Third:C,1)", output_1_3, context)
         self.assertScriptResultEquals("$slice(First:A; Second:B; Third:C,1,)", output_1_3, context)
         self.assertScriptResultEquals("$slice(First:A; Second:B; Third:C,,)", output_0_3, context)
         # Tests with invalid inputs (end < start)
@@ -1283,13 +1366,11 @@ class ScriptParserTest(PicardTestCase):
         # Tests with separator override
         self.assertScriptResultEquals("$slice(First:A; Second:B; Third:C,1,3,:)", alternate_output, context)
         # Tests with invalid number of arguments
-        areg = r"^\d+:\d+:\$slice: Wrong number of arguments for \$slice: Expected between 3 and 4, "
+        areg = r"^\d+:\d+:\$slice: Wrong number of arguments for \$slice: Expected between 2 and 4, "
         with self.assertRaisesRegex(ScriptError, areg):
             self.parser.eval("$slice()")
         with self.assertRaisesRegex(ScriptError, areg):
             self.parser.eval("$slice(abc; def)")
-        with self.assertRaisesRegex(ScriptError, areg):
-            self.parser.eval("$slice(abc; def,0)")
         with self.assertRaisesRegex(ScriptError, areg):
             self.parser.eval("$slice(abc; def),0,1,:,extra")
 
@@ -1495,3 +1576,212 @@ class ScriptParserTest(PicardTestCase):
             self.parser.eval("$reversemulti()")
         with self.assertRaisesRegex(ScriptError, areg):
             self.parser.eval("$reversemulti(B:AB; D:C; E:D; A:A; C:X,:,extra)")
+
+    def test_cmd_unique(self):
+        context = Metadata()
+        context["foo"] = ['a', 'A', 'B', 'b', 'cd', 'Cd', 'cD', 'CD', 'a', 'A', 'b']
+        context["bar"] = "a; A; B; b; cd; Cd; cD; CD; a; A; b"
+        # Tests with context
+        self.assertScriptResultEquals("$unique(%foo%)", "A; CD; b", context)
+        self.assertScriptResultEquals("$unique(%bar%)", "a; A; B; b; cd; Cd; cD; CD; a; A; b", context)
+        # Tests with static inputs
+        self.assertScriptResultEquals("$unique(a; A; B; b; cd; Cd; cD; CD; a; A; b)", "A; CD; b", context)
+        # Tests with separator override
+        self.assertScriptResultEquals("$unique(a: A: B: b: cd: Cd: cD: CD: a: A: b,,: )", "A: CD: b", context)
+        # Tests with case-sensitive comparison
+        self.assertScriptResultEquals("$unique(%foo%,1)", "A; B; CD; Cd; a; b; cD; cd", context)
+        # Tests with missing inputs
+        self.assertScriptResultEquals("$unique(,)", "", context)
+        self.assertScriptResultEquals("$unique(,,)", "", context)
+        self.assertScriptResultEquals("$unique(,:)", "", context)
+        # Tests with invalid number of arguments
+        areg = r"^\d+:\d+:\$unique: Wrong number of arguments for \$unique: Expected between 1 and 3, "
+        with self.assertRaisesRegex(ScriptError, areg):
+            self.parser.eval("$unique()")
+        with self.assertRaisesRegex(ScriptError, areg):
+            self.parser.eval("$unique(B:AB; D:C; E:D; A:A; C:X,1,:,extra)")
+
+    def test_cmd_countryname(self):
+        context = Metadata()
+        context["foo"] = "ca"
+        context["bar"] = ""
+        context["baz"] = "INVALID"
+
+        # Ensure that `builtins` contains a `gettext_countries` attribute to avoid an `AttributeError`
+        # exception when mocking the function, in case the `picard.i18n` module has not been loaded.
+        # This is required by the $countryname() function in order to translate the country names.
+        if not hasattr(builtins, 'gettext_countries'):
+            builtins.__dict__['gettext_countries'] = None
+
+        # Mock function to simulate English locale.
+        def mock_gettext_countries_en(arg):
+            return arg
+
+        # Mock function to simulate Russian locale.
+        def mock_gettext_countries_ru(arg):
+            return "Канада" if arg == 'Canada' else arg
+
+        # Test with Russian locale
+        with mock.patch('builtins.gettext_countries', mock_gettext_countries_ru):
+            self.assertScriptResultEquals("$countryname(ca)", "Canada", context)
+            self.assertScriptResultEquals("$countryname(ca,)", "Canada", context)
+            self.assertScriptResultEquals("$countryname(ca, )", "Канада", context)
+            self.assertScriptResultEquals("$countryname(ca,yes)", "Канада", context)
+            self.assertScriptResultEquals("$countryname(INVALID,yes)", "", context)
+            # Test for unknown translation of correct code
+            self.assertScriptResultEquals("$countryname(fr,yes)", "France", context)
+
+        # Reset locale to English for remaining tests
+        with mock.patch('builtins.gettext_countries', mock_gettext_countries_en):
+            self.assertScriptResultEquals("$countryname(ca,)", "Canada", context)
+            self.assertScriptResultEquals("$countryname(ca,yes)", "Canada", context)
+            self.assertScriptResultEquals("$countryname(ca)", "Canada", context)
+            self.assertScriptResultEquals("$countryname(CA)", "Canada", context)
+            self.assertScriptResultEquals("$countryname(%foo%)", "Canada", context)
+            self.assertScriptResultEquals("$countryname(%bar%)", "", context)
+            self.assertScriptResultEquals("$countryname(%baz%)", "", context)
+            self.assertScriptResultEquals("$countryname(INVALID)", "", context)
+
+        # Tests with invalid number of arguments
+        areg = r"^\d+:\d+:\$countryname: Wrong number of arguments for \$countryname: Expected between 1 and 2, "
+        with self.assertRaisesRegex(ScriptError, areg):
+            self.parser.eval("$countryname()")
+        with self.assertRaisesRegex(ScriptError, areg):
+            self.parser.eval("$countryname(CA,,Extra)")
+
+    def test_cmd_year(self):
+        context = Metadata()
+        context["foo"] = "07.21.2021"
+        context["bar"] = "mdy"
+
+        # Test with default values
+        self.assertScriptResultEquals("$year(2021 07 21)", "2021", context)
+        self.assertScriptResultEquals("$year(2021.07.21)", "2021", context)
+        self.assertScriptResultEquals("$year(2021-07-21)", "2021", context)
+        self.assertScriptResultEquals("$year(21-07-21)", "21", context)
+
+        # Test with overrides specified
+        self.assertScriptResultEquals("$year(%foo%,%bar%)", "2021", context)
+
+        # Test with invalid overrides
+        self.assertScriptResultEquals("$year(2021-07-21,myd)", "2021", context)
+
+        # Test missing elements
+        self.assertScriptResultEquals("$year(,)", "", context)
+        self.assertScriptResultEquals("$year(07-21,mdy)", "", context)
+        self.assertScriptResultEquals("$year(21-07,dmy)", "", context)
+
+        # Tests with invalid number of arguments
+        areg = r"^\d+:\d+:\$year: Wrong number of arguments for \$year: Expected between 1 and 2, "
+        with self.assertRaisesRegex(ScriptError, areg):
+            self.parser.eval("$year()")
+        with self.assertRaisesRegex(ScriptError, areg):
+            self.parser.eval("$year(2021-07-21,,)")
+
+    def test_cmd_month(self):
+        context = Metadata()
+        context["foo"] = "07.21.2021"
+        context["bar"] = "mdy"
+
+        # Test with default values
+        self.assertScriptResultEquals("$month(2021 07 21)", "07", context)
+        self.assertScriptResultEquals("$month(2021.07.21)", "07", context)
+        self.assertScriptResultEquals("$month(2021-07-21)", "07", context)
+        self.assertScriptResultEquals("$month(2021-7-21)", "7", context)
+
+        # Test with overrides specified
+        self.assertScriptResultEquals("$month(%foo%,%bar%)", "07", context)
+
+        # Test with invalid overrides
+        self.assertScriptResultEquals("$month(2021-07-21,myd)", "07", context)
+
+        # Test missing elements
+        self.assertScriptResultEquals("$month(,)", "", context)
+        self.assertScriptResultEquals("$month(-21-2021,mdy)", "", context)
+
+        # Tests with invalid number of arguments
+        areg = r"^\d+:\d+:\$month: Wrong number of arguments for \$month: Expected between 1 and 2, "
+        with self.assertRaisesRegex(ScriptError, areg):
+            self.parser.eval("$month()")
+        with self.assertRaisesRegex(ScriptError, areg):
+            self.parser.eval("$month(2021-07-21,,)")
+
+    def test_cmd_day(self):
+        context = Metadata()
+        context["foo"] = "07.21.2021"
+        context["bar"] = "mdy"
+
+        # Test with default values
+        self.assertScriptResultEquals("$day(2021 07 21)", "21", context)
+        self.assertScriptResultEquals("$day(2021.07.21)", "21", context)
+        self.assertScriptResultEquals("$day(2021-07-21)", "21", context)
+        self.assertScriptResultEquals("$day(2021-07-2)", "2", context)
+
+        # Test with overrides specified
+        self.assertScriptResultEquals("$day(%foo%,%bar%)", "21", context)
+
+        # Test with invalid overrides
+        self.assertScriptResultEquals("$day(2021-07-21,myd)", "21", context)
+
+        # Test missing elements
+        self.assertScriptResultEquals("$day(,)", "", context)
+        self.assertScriptResultEquals("$day(-07-2021,dmy)", "", context)
+
+        # Tests with invalid number of arguments
+        areg = r"^\d+:\d+:\$day: Wrong number of arguments for \$day: Expected between 1 and 2, "
+        with self.assertRaisesRegex(ScriptError, areg):
+            self.parser.eval("$day()")
+        with self.assertRaisesRegex(ScriptError, areg):
+            self.parser.eval("$day(2021-07-21,,)")
+
+    def test_cmd_dateformat(self):
+        context = Metadata()
+        context["foo"] = "07.21.2021"
+        context["bar"] = "mdy"
+        context["format"] = "%Y.%m.%d"
+
+        # Test with default values
+        self.assertScriptResultEquals("$dateformat(2021 07 21)", "2021-07-21", context)
+        self.assertScriptResultEquals("$dateformat(2021.07.21)", "2021-07-21", context)
+        self.assertScriptResultEquals("$dateformat(2021-07-21)", "2021-07-21", context)
+        self.assertScriptResultEquals("$dateformat(2021-7-21)", "2021-07-21", context)
+
+        # Test with overrides specified
+        self.assertScriptResultEquals("$dateformat(%foo%,%format%,%bar%)", "2021.07.21", context)
+
+        # Test with invalid overrides
+        self.assertScriptResultEquals("$dateformat(2021-07-21,,myd)", "2021-07-21", context)
+        self.assertScriptResultEquals("$dateformat(2021-07-21,,dmy)", "", context)
+        self.assertScriptResultEquals("$dateformat(2021-07-21,,mdy)", "", context)
+        self.assertScriptResultEquals("$dateformat(2021-July-21)", "", context)
+        self.assertScriptResultEquals("$dateformat(2021)", "", context)
+        self.assertScriptResultEquals("$dateformat(2021-07)", "", context)
+
+        # Test missing elements
+        self.assertScriptResultEquals("$dateformat(,)", "", context)
+
+        # Tests with invalid number of arguments
+        areg = r"^\d+:\d+:\$dateformat: Wrong number of arguments for \$dateformat: Expected between 1 and 3, "
+        with self.assertRaisesRegex(ScriptError, areg):
+            self.parser.eval("$dateformat()")
+        with self.assertRaisesRegex(ScriptError, areg):
+            self.parser.eval("$dateformat(2021-07-21,,,)")
+
+    def test_cmd_is_multi(self):
+        context = Metadata()
+        context["foo"] = "a; b; c"
+        context["bar"] = ""
+
+        self.assertScriptResultEquals("$is_multi(%foo%)", "", context)
+        self.assertScriptResultEquals("$is_multi(%bar%)", "", context)
+        self.assertScriptResultEquals("$setmulti(baz,a)$is_multi(%baz%)", "", context)
+        self.assertScriptResultEquals("$setmulti(baz,a; b; c)$is_multi(%baz%)", "1", context)
+        self.assertScriptResultEquals("$is_multi(a; b; c)", "1", context)
+        self.assertScriptResultEquals("$is_multi(a)", "", context)
+
+        # Tests with invalid number of arguments
+        areg = r"^\d+:\d+:\$is_multi: Wrong number of arguments for \$is_multi: Expected exactly 1, "
+        with self.assertRaisesRegex(ScriptError, areg):
+            self.parser.eval("$is_multi()")
+        with self.assertRaisesRegex(ScriptError, areg):
+            self.parser.eval("$is_multi(a,)")
