@@ -3,7 +3,7 @@
 # Picard, the next-generation MusicBrainz tagger
 #
 # Copyright (C) 2018-2020 Philipp Wolfer
-# Copyright (C) 2019 Laurent Monin
+# Copyright (C) 2019-2020 Laurent Monin
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -53,7 +53,7 @@ class ScriptToFilenameTest(PicardTestCase):
 
     def setUp(self):
         super().setUp()
-        config.setting = settings.copy()
+        self.set_config_values(settings)
 
     def test_plain_filename(self):
         metadata = Metadata()
