@@ -2,7 +2,8 @@
 #
 # Picard, the next-generation MusicBrainz tagger
 #
-# Copyright (C) 2019 Laurent Monin
+# Copyright (C) 2019-2020 Laurent Monin
+# Copyright (C) 2020 Philipp Wolfer
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -28,8 +29,8 @@ from picard.util.settingsoverride import SettingsOverride
 class SettingsOverrideTest(PicardTestCase):
 
     def setUp(self):
-        self.config = {'key1': 'origval1', 'key2': 'origval2'}
-        config.setting = self.config.copy()
+        super().setUp()
+        self.set_config_values({'key1': 'origval1', 'key2': 'origval2'})
         self.new_settings = {'key1': 'newval2'}
 
     def test_read_orig_settings(self):
