@@ -2,8 +2,9 @@
 #
 # Picard, the next-generation MusicBrainz tagger
 #
-# Copyright (C) 2019 Laurent Monin
 # Copyright (C) 2019 Wieland Hoffmann
+# Copyright (C) 2019-2021 Laurent Monin
+# Copyright (C) 2020 Philipp Wolfer
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -167,4 +168,5 @@ class TagGenreFilterTest(PicardTestCase):
 
     def test_filter_method(self):
         tag_filter = TagGenreFilter("-a*")
-        self.assertEqual(['bx', 'by'], tag_filter.filter(["ax", "bx", "ay", "by"]))
+        result = list(tag_filter.filter([("ax", 1), ("bx", 2), ("ay", 3), ("by", 4)]))
+        self.assertEqual([('bx', 2), ('by', 4)], result)
