@@ -5,8 +5,8 @@
 # Copyright (C) 2013-2014 Ionuț Ciocîrlan
 # Copyright (C) 2016 Sambhav Kothari
 # Copyright (C) 2018 Wieland Hoffmann
-# Copyright (C) 2018-2019 Laurent Monin
-# Copyright (C) 2019-2020 Philipp Wolfer
+# Copyright (C) 2018-2020 Laurent Monin
+# Copyright (C) 2019-2021 Philipp Wolfer
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -40,8 +40,11 @@ from picard.const.sys import (
 )
 from picard.util.filenaming import (
     WinPathTooLong,
+    get_available_filename,
+    make_save_path,
     make_short_filename,
     move_ensure_casing,
+    replace_extension,
     samefile_different_casing,
 )
 
@@ -57,44 +60,44 @@ class ShortFilenameTest(PicardTestCase):
             self.max_len = os.statvfs("/").f_namemax
         super().__init__(*args, **kwargs)
 
-    @unittest.skipUnless(IS_WIN or IS_MACOS, "windows / os x test")
+    @unittest.skipUnless(IS_WIN or IS_MACOS, "windows / macOS test")
     def test_bmp_unicode_on_unicode_fs(self):
-        char = u"\N{LATIN SMALL LETTER SHARP S}"
+        char = "\N{LATIN SMALL LETTER SHARP S}"
         fn = make_short_filename(self.root, os.path.join(*[char * 120] * 2))
         self.assertEqual(fn, os.path.join(*[char * 120] * 2))
 
     @unittest.skipUnless(not IS_WIN and not IS_MACOS, "non-windows, non-osx test")
     def test_bmp_unicode_on_nix(self):
-        char = u"\N{LATIN SMALL LETTER SHARP S}"
+        char = "\N{LATIN SMALL LETTER SHARP S}"
         max_len = self.max_len
         divisor = len(char.encode(sys.getfilesystemencoding()))
         fn = make_short_filename(self.root, os.path.join(*[char * 200] * 2))
         self.assertEqual(fn, os.path.join(*[char * (max_len // divisor)] * 2))
 
-    @unittest.skipUnless(IS_MACOS, "os x test")
+    @unittest.skipUnless(IS_MACOS, "macOS test")
     def test_precomposed_unicode_on_osx(self):
-        char = u"\N{LATIN SMALL LETTER A WITH BREVE}"
+        char = "\N{LATIN SMALL LETTER A WITH BREVE}"
         max_len = self.max_len
         fn = make_short_filename(self.root, os.path.join(*[char * 200] * 2))
         self.assertEqual(fn, os.path.join(*[char * (max_len // 2)] * 2))
 
     @unittest.skipUnless(IS_WIN, "windows test")
     def test_nonbmp_unicode_on_windows(self):
-        char = u"\N{MUSICAL SYMBOL G CLEF}"
+        char = "\N{MUSICAL SYMBOL G CLEF}"
         remaining = 259 - (3 + 10 + 1 + 200 + 1)
         fn = make_short_filename(self.root, os.path.join(*[char * 100] * 2))
         self.assertEqual(fn, os.path.join(char * 100, char * (remaining // 2)))
 
-    @unittest.skipUnless(IS_MACOS, "os x test")
+    @unittest.skipUnless(IS_MACOS, "macOS test")
     def test_nonbmp_unicode_on_osx(self):
-        char = u"\N{MUSICAL SYMBOL G CLEF}"
+        char = "\N{MUSICAL SYMBOL G CLEF}"
         max_len = self.max_len
         fn = make_short_filename(self.root, os.path.join(*[char * 200] * 2))
         self.assertEqual(fn, os.path.join(*[char * (max_len // 2)] * 2))
 
     @unittest.skipUnless(not IS_WIN and not IS_MACOS, "non-windows, non-osx test")
     def test_nonbmp_unicode_on_nix(self):
-        char = u"\N{MUSICAL SYMBOL G CLEF}"
+        char = "\N{MUSICAL SYMBOL G CLEF}"
         max_len = self.max_len
         divisor = len(char.encode(sys.getfilesystemencoding()))
         fn = make_short_filename(self.root, os.path.join(*[char * 100] * 2))
@@ -102,7 +105,7 @@ class ShortFilenameTest(PicardTestCase):
 
     @unittest.skipUnless(not IS_WIN and not IS_MACOS, "non-windows, non-osx test")
     def test_nonbmp_unicode_on_nix_with_windows_compat(self):
-        char = u"\N{MUSICAL SYMBOL G CLEF}"
+        char = "\N{MUSICAL SYMBOL G CLEF}"
         max_len = self.max_len
         remaining = 259 - (3 + 10 + 1 + 200 + 1)
         divisor = len(char.encode(sys.getfilesystemencoding()))
@@ -182,3 +185,72 @@ class MoveEnsureCasingTest(PicardTestCase):
             move_ensure_casing(file_path, target_path)
             files = os.listdir(d)
             self.assertIn('FOO', files)
+
+
+class MakeSavePathTest(PicardTestCase):
+
+    def test_replace_trailing_dots(self):
+        path = 'foo./bar.'
+        self.assertEqual(path, make_save_path(path))
+        self.assertEqual('foo_/bar_', make_save_path(path, win_compat=True))
+
+    def test_replace_leading_dots(self):
+        path = '.foo/.bar'
+        self.assertEqual('_foo/_bar', make_save_path(path))
+
+    def test_decompose_precomposed_chars(self):
+        path = 'foo/\u00E9bar'  # é
+        self.assertEqual('foo/\u0065\u0301bar', make_save_path(path, mac_compat=True))
+
+
+class GetAvailableFilenameTest(PicardTestCase):
+
+    def _add_number(self, filename, number):
+        name, ext = os.path.splitext(filename)
+        return '%s (%i)%s' % (name, number, ext)
+
+    def test_append_number(self):
+        with NamedTemporaryFile(prefix='foo', suffix='.mp3') as f:
+            new_filename = get_available_filename(f.name)
+            self.assertEqual(self._add_number(f.name, 1), new_filename)
+
+    def test_do_not_append_number_on_same_file(self):
+        with NamedTemporaryFile(prefix='foo', suffix='.mp3') as f:
+            new_filename = get_available_filename(f.name, f.name)
+            self.assertEqual(f.name, new_filename)
+
+    def test_handle_non_existant_old_path(self):
+        with NamedTemporaryFile(prefix='foo', suffix='.mp3') as f:
+            new_filename = get_available_filename(f.name, '/foo/old.mp3')
+            self.assertEqual(self._add_number(f.name, 1), new_filename)
+
+    def test_append_additional_numbers(self):
+        with TemporaryDirectory() as d:
+            expected_number = 3
+            oldname = os.path.join(d, 'bar.mp3')
+            open(oldname, 'a').close()
+            filename = os.path.join(d, 'foo.mp3')
+            open(filename, 'a').close()
+            for i in range(1, expected_number):
+                open(self._add_number(filename, i), 'a').close()
+            new_filename = get_available_filename(filename, oldname)
+            self.assertEqual(self._add_number(filename, expected_number), new_filename)
+
+    def test_reuse_existing_number(self):
+        with TemporaryDirectory() as d:
+            expected_number = 2
+            filename = os.path.join(d, 'foo.mp3')
+            open(filename, 'a').close()
+            for i in range(1, 3):
+                open(self._add_number(filename, i), 'a').close()
+            oldname = self._add_number(filename, expected_number)
+            new_filename = get_available_filename(filename, oldname)
+            self.assertEqual(self._add_number(filename, expected_number), new_filename)
+
+
+class ReplaceExtensionTest(PicardTestCase):
+
+    def test_replace(self):
+        self.assertEqual('foo/bar.wvc', replace_extension('foo/bar.wv', '.wvc'))
+        self.assertEqual('foo/bar.wvc', replace_extension('foo/bar.wv', 'wvc'))
+        self.assertEqual('foo/bar.wvc', replace_extension('foo/bar', 'wvc'))
